// Este programa implementa el algoritmo de Prim sobre una grfica almacenada en listas de adyacencia.
// El algoritmo de Prim calcula un rbol abarcador de costo mnimo, el cual conecta todos los vrtices de la grfica de modo que la suma de las aristas usadas se minimice.
// Este algoritmo funciona sin importar desde qu vertice se comience la construccin del rbol (el rbol puede resultar diferente, pero tiene el mismo costo).
// El algoritmo de Prim mete en una cola de prioridad la informacin de las aristas que descubre conforme el rbol crece y saca de la cola la informacin que corresponde a
// la arista ms barata. Si poner la arista sacada creara un ciclo, entonces la ignora y saca la siguiente.
#include <algorithm>
#include <functional>
#include <climits>
#include <iostream>
#include <queue>
#include <utility>
#include <tuple>              // generalizacin de pair
#include <vector>
using namespace std;

template<typename T>
using min_heap = priority_queue<T, vector<T>, greater<T>>;

vector<int> prim(int i, vector<pair<int, int>> adyacencia[], int v)
{
   vector<int> predecesores(v, -1);       // el predecesor de un vrtice es el vecino al que nos conectamos para incorporarnos al rbol; -1 ndica que an no estamos en el rbol
   min_heap<tuple<int, int, int>> cola;   // guardaremos tripletas (costo_arista, vrtice, predecesor) en la cola, las cuales denotan una posible manera de incorporar un vrtice al rbol
   cola.push({ 0, i, i });                // de la cola ir saliendo la tripleta con la arista ms barata; como caso especial, el vrtice inicial puede incorporarse a costo 0 por s mismo

   do {
      auto [costo, actual, previo] = cola.top( );    // C++17!
      cola.pop( );
      if (predecesores[actual] != -1) {              // si el vrtice ya estaba incorporado al rbol, lo ignoramos
         continue;
      }

      predecesores[actual] = previo;                     // si el vrtice no estaba incorporado al rbol, registramos que lo incorporamos usando el predecesor sugerido
      for (auto [costo, vecino] : adyacencia[actual]) {  // C++17! iterativamente inspeccionamos a sus vecinos
         cola.push({ costo, vecino, actual });           // nuestro vecino podra incorporarse con la arista que estamos viendo en este momento y el predecesor seramos nosotros
      }
   } while (!cola.empty( ));

   return predecesores;
}

int main( )
{
   int v, e;
   cin >> v >> e;

   vector<pair<int, int>> adyacencia[v];
   for (int i = 0; i < e; ++i) {
      int x, y, c;
      cin >> x >> y >> c;
      adyacencia[x].push_back({ c, y });
      adyacencia[y].push_back({ c, x });
   }

   int inicial;
   cin >> inicial;

   auto predecesores = prim(inicial, adyacencia, v);
   for (int i = 0; i < v; ++i) {
      cout << i << ": se conecta con " << predecesores[i] << "\n";
   }

   int costo = 0;
   for (int i = 0; i < v; ++i) {
      costo += (i == inicial ? 0 : find_if(adyacencia[i].begin( ), adyacencia[i].end( ), [&](const pair<int, int>& p) {
         return p.second == predecesores[i];
      })->first);
   }
   cout << "El costo total del arbol es " << costo << "\n";
}
